import torch.utils.data as data
import os
import os.path
#from scipy.ndimage import imread
import pyexr
import numpy as np
import torch


def get_flow(filename):
    with open(filename, 'rb') as f:
        magic = np.fromfile(f, np.float32, count=1)
        if 202021.25 != magic:
            print ('Magic number incorrect. Invalid .flo file')
        else:
            w = np.fromfile(f, np.int32, count=1)
            h = np.fromfile(f, np.int32, count=1)
            data = np.fromfile(f, np.float32, count= int(2*w*h))
            # Reshape data into 3D array (columns, rows, bands)
            data2D = np.resize(data, (h[0], w[0], 2))
            #data2D = np.transpose(data2D,[0, 3,1,2])
            return data2D

def frame_loader(root, path, path_flow):
    features = []
    gts = []
    for image in path:
        current_frame = pyexr.open(image)
        current_feature = [current_frame.get("diffuse_dir"), current_frame.get("normal"), current_frame.get("depth"), current_frame.get("albedo")]
        current_feature = np.concatenate(current_feature, axis = 2)
        current_feature = np.transpose(current_feature, (2, 0, 1))[np.newaxis,:,:,:]
        features.append(current_feature)

        gt_feature = current_frame.get("diffuse")
        gt_feature = np.transpose(gt_feature, (2, 0, 1))[np.newaxis,:,:,:]
        gts.append(gt_feature)

    features = np.concatenate(features, axis = 0)
    gts = np.concatenate(gts, axis = 0)

    flows = []
    for flow in path_flow:
        current_flow = get_flow(flow)[np.newaxis,:,:,:]
        flows.append(current_flow)
    flows = np.concatenate(flows, axis = 0)

    return features, gts, flows


class ListDataset(data.Dataset):
    def __init__(self, root, path_list, transform=None, target_transform=None,
                 co_transform=None, loader=frame_loader):

        self.root = root
        self.path_list = path_list
        self.transform = transform
        self.target_transform = target_transform
        self.co_transform = co_transform
        self.loader = loader

    def __getitem__(self, index):
        image_list, flow_list = self.path_list[index]

        features, gts, flows = self.loader(self.root, image_list, flow_list)
        features = torch.from_numpy(features).float()
        gts = torch.from_numpy(gts).float()
        flows = torch.from_numpy(flows).float()

        return  features, gts, flows

    def __len__(self):
        return len(self.path_list)
